<?

 /*************************************************************************************
 * Filename: modules/topics.php
 *
 * This file is part of the DocIT JAS plugin.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The DocIT JAS plugin is free software under the terms of the
 * GNU General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the plugin or http://jas.junerules.com/plugins/docit/
 *************************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_docit_topics extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->data->title="DocIT topics";

   $this->data->defaultFields();

   $this->data->addField("topic", "text", "all", "title", 1, 255, false, "Topic");
   $this->data->addField("content", "text", "all", "markup", 0, 65536, false, "Content");


   $this->data->addField("published", "boolean", "all", "", 0, false, true, "Published");

   $this->data->addField("parent", "nr", "all", "", 0, false, 0, "Parent topic");
   $this->data->addReference("parent", "docit_topics", "nr", "*:0");

   $this->data->addField("name", "regexp", "all", "", false, 25, "", "Topic short name");
   $this->data->regexps("name", "/^([A-Za-z0-9\_]+)$/");

   $this->data->addField("ordernr", "nr", "sql", "order", false, false, 0, "Order number", "Leave: 0 for automatic number");

   $this->data->addField("path", "text", "sql;show", "", 0, 65536, false, "Path");
   $this->data->addField("depth", "nr", "sql;show", "", 0, false, false, "Path depth");


   $this->data->addReference("subtopics", "docit_topics", "parent", "0:*", "nr");
   if(isset($this->data->field["subtopics"]))
    $this->data->field["subtopics"]["properties"].=";hidden";

   $this->data->addReference("comments", "docit_comments", "docittopicnr", "1:*", "nr");
   if(isset($this->data->field["comments"]))
    $this->data->field["comments"]["properties"].=";hidden";


   $this->data->addFullTextIndex("fulltext", "topic", "content");
   $this->data->addUnique("topic", "name", "parent");

   $this->data->group("Info", "published", "parent", "name", "ordernr", "path", "depth", "comments", "subtopics");

   $this->data->sortOrder("ordernr", "topic");
  }

  function addPreSubmitErrorCheck(){return $this->setShortNamePathDepth();}
  function editPreSubmitErrorCheck(){return $this->setShortNamePathDepth();}

  function setShortNamePathDepth()
  {
   //name
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!$this->data->getValue("name", $c)&&$this->data->getValue("topic", $c))
    {
     $name=str_replace(" ", "_", $this->data->getValue("topic", $c)); //replace spaces
     $name=preg_replace("/[^A-Za-z0-9\_]/i", "", $name);
     $this->data->setValue("name", $name, $c);
    }
   }

   //path & depth
   $cp=0;
   $d=Data::cache($this->data->module, "view", "item");
   $d->setShowFields("path");
   $d->clearValues();

   for($c=0;$c<$this->data->vmax;$c++) //get parents
   {
    if($this->data->getValue("parent", $c))
    {
     $d->setValue("nr", $this->data->getValue("parent", $c), $cp);
     $d->setOperant("nr", "OR", $cp);
     $cp++;
    }
   }

   if($cp&&!$d->get())
   {
    $this->data->e->add("Unable to get parents for setting path & depth", "error");
    return false;
   }

   for($c=0;$c<$this->data->vmax;$c++) //set path & depth
   {
    if($this->data->getValue("name", $c))
    {
     $d->reset();
     if($cp&&$d->search("nr", $this->data->getValue("parent", $c)))
      $this->data->setValue("path", $d->v["path"]."/".$this->data->getValue("name", $c), $c);
     else
      $this->data->setValue("path", $this->data->getValue("name", $c), $c);

     $this->data->setValue("depth", substr_count($this->data->getValue("path", $c), "/")+1, $c);
    }
   }

   return true;
  }

  function deletePreSubmit()
  {
   $d=Data::cache($this->data->module, "view", "all");

   for($c=0;$c<$this->data->vmax;$c++) //set path & depth
   {
    $d->clearValues();
    $d->setValue("parent", $this->data->getValue("nr", $c));

    if($d->get(false, true)&&$d->getValue("show_count"))
    {
     $this->data->e->add("Topic has sub-topics, refusing to delete", "error", "Entry: ".($c+1));
     return false;
    }
   }

   return true;
  }


 }
?>
